!= 
!
! Authors::   Yuta Kawai
! Version::   $Id: grid_test.f90,v 1.2 2011-05-17 15:34:20 ykawai Exp $
! Tag Name::  $Name:  $
! Copyright:: Copyright (C) GFD Dennou Club, 2011-2012. All rights reserved.
! License::   See COPYRIGHT[link:../../COPYRIGHT]
!
program main
  use dc_types
  use IcGrid2D_FVM_Manager
  use IcGrid2D_FVM_Builder
  implicit none
  
  type(IcGrid2D_FVM) :: ic_grid
  integer :: div_level = 5
  real(DP) :: ic_radius = 1.0d0

  !
  !
  write(*,*) '= grid_test 1'
  write(*,*) '* construct_icosahedral_grid(1)'
  call construct_icosahedral_grid(GTYPE_STDGRID, div_level, ic_radius, ic_grid)
  call IcGrid2D_FVM_Final(ic_grid)

  !
  !
  write(*,*) '= grid_test 2'
  write(*,*) '* construct_icosahedral_grid(2)'
  call IcGrid2D_FVM_Init(ic_grid, div_level, 1.0d0 )
  call malloc_GP_array(ic_grid)
  call malloc_CV_array(ic_grid)
  call construct_icosahedral_grid(GTYPE_STDGRID, ic_grid)

  
  ! netCDF 書き込みテスト
  !
  write(*,*) '= grid_test 3'
  write(*,*) '* Writes icosahedral grid data into NetCDF file.'
  call grid_test3(ic_grid)

  ! netCDF 読み込みテスト
  !
  write(*,*) '= grid_test 4'
  write(*,*) '* Reads icosahedral grid data from NetCDF file.'
  call grid_test4(ic_grid)

  ! 後片付け
  write(*,*) '= grid_test 4'
  write(*,*) '* Calculates the vertex and area of control volumes'

  call calc_CV_vertex(ic_grid)
  call calc_CV_Area(ic_grid)
  call IcGrid2D_FVM_Final(ic_grid)

contains
subroutine grid_test3(icgrid)
  use IcGrid_ncWriter_mod

  type(IcGrid2D_FVM), intent(in) :: icgrid

  type(IcGrid_ncWriter) :: writer
  integer :: EMin, EMax, EffSize

  EMin = get_EffSize_Min(icgrid)
  EMax = get_EffSize_Max(icgrid)
  EffSize = EMax - EMin + 1

  call IcGrid_ncWriter_Init(writer, icgrid)
  call open_ncFile(writer, 'icgrid.nc')
  call ncdef_GridData(writer)
  call end_ncdef(writer)

  call write_GridData(writer)
  call close_ncFile(writer)

  write(*,*) 'succeed to write netCDF file'

end subroutine grid_test3

subroutine grid_test4(icgrid)
  use IcGrid_ncReader_mod

  type(IcGrid2D_FVM), intent(inout) :: icgrid

  type(IcGrid_ncReader) :: reader
  integer :: EMin, EMax, EffSize

  EMin = get_EffSize_Min(icgrid)
  EMax = get_EffSize_Max(icgrid)
  EffSize = EMax - EMin + 1

  call IcGrid_ncReader_Init(reader, icgrid)
  call open_ncFile(reader, 'icgrid.nc')
  call read_GridData(reader)
  call close_ncFile(reader)

  write(*,*) 'succeed to read netCDF file'

end subroutine grid_test4

end program
 
